]> git.sesse.net Git - nageru/blobdiff - mixer.h
Hook up the channel click events.
[nageru] / mixer.h
diff --git a/mixer.h b/mixer.h
index c4697310ec95757fecbfacea767ed887ba821e03..c6366fdec1a65d5e8f7e909ed788e0c6119d8c73 100644 (file)
--- a/mixer.h
+++ b/mixer.h
@@ -6,6 +6,7 @@
 #include <epoxy/gl.h>
 #undef Success
 #include <movit/effect_chain.h>
+#include <movit/flat_input.h>
 #include <functional>
 
 #include "bmusb.h"
@@ -13,6 +14,7 @@
 #include "pbo_frame_allocator.h"
 #include "ref_counted_frame.h"
 #include "ref_counted_gl_sync.h"
+#include "theme.h"
 
 #define NUM_CARDS 2
 
@@ -30,22 +32,39 @@ public:
        void start();
        void quit();
 
-       enum Source {
-               SOURCE_INPUT1,
-               SOURCE_INPUT2,
-               SOURCE_SBS,
-       };
-       void cut(Source source);
+       void transition_clicked(int transition_num);
+       void channel_clicked(int preview_num);
 
        enum Output {
                OUTPUT_LIVE = 0,
                OUTPUT_PREVIEW,
+               OUTPUT_INPUT0,
+               OUTPUT_INPUT1,
+               OUTPUT_INPUT2,
+               OUTPUT_INPUT3,
                NUM_OUTPUTS
        };
 
        struct DisplayFrame {
-               GLuint texnum;
-               RefCountedGLsync ready_fence;  // Asserted when the texture is done rendering.
+               // The chain for rendering this frame. To render a display frame,
+               // first wait for <ready_fence>, then call <setup_chain>
+               // to wire up all the inputs, and then finally call
+               // chain->render_to_screen() or similar.
+               movit::EffectChain *chain;
+               std::function<void()> setup_chain;
+
+               // Asserted when all the inputs are ready; you cannot render the chain
+               // before this.
+               RefCountedGLsync ready_fence;
+
+               // Holds on to all the input frames needed for this display frame,
+               // so they are not released while still rendering.
+               std::vector<RefCountedFrame> input_frames;
+
+               // Textures that should be released back to the resource pool
+               // when this frame disappears, if any.
+               // TODO: Refcount these as well?
+               std::vector<GLuint> temp_textures;
        };
        // Implicitly frees the previous one if there's a new frame available.
        bool get_display_frame(Output output, DisplayFrame *frame) {
@@ -58,12 +77,6 @@ public:
                output_channel[output].set_frame_ready_callback(callback);
        }
 
-       // Ignored for OUTPUT_LIVE.
-       void set_preview_size(Output output, int width, int height)
-       {
-               output_channel[output].set_size(width, height);
-       }
-
 private:
        void bm_frame(int card_index, uint16_t timecode,
                FrameAllocator::Frame video_frame, size_t video_offset, uint16_t video_format,
@@ -75,20 +88,14 @@ private:
 
        QSurface *mixer_surface, *h264_encoder_surface;
        std::unique_ptr<movit::ResourcePool> resource_pool;
-       std::unique_ptr<movit::EffectChain> chain;
-       std::unique_ptr<movit::EffectChain> preview_chain;
+       std::unique_ptr<Theme> theme;
+       std::unique_ptr<movit::EffectChain> display_chain;
        GLuint cbcr_program_num;  // Owned by <resource_pool>.
        std::unique_ptr<H264Encoder> h264_encoder;
 
-       // Effects part of <chain>. Owned by <chain>.
-       movit::YCbCrInput *input[NUM_CARDS];
-       movit::Effect *resample_effect, *resample2_effect;
-       movit::Effect *padding_effect, *padding2_effect;
-
-       // Effects part of <preview_chain>. Owned by <preview_chain>.
-       movit::YCbCrInput *preview_input;
+       // Effects part of <display_chain>. Owned by <display_chain>.
+       movit::FlatInput *display_input;
 
-       Source current_source = SOURCE_INPUT1;
        int frame = 0;
 
        std::mutex bmusb_mutex;
@@ -96,8 +103,7 @@ private:
                BMUSBCapture *usb;
                std::unique_ptr<PBOFrameAllocator> frame_allocator;
 
-               // Threading stuff
-               bool thread_initialized = false;
+               // Stuff for the OpenGL context (for texture uploading).
                QSurface *surface;
                QOpenGLContext *context;
 
@@ -112,10 +118,10 @@ private:
 
        class OutputChannel {
        public:
-               void output_frame(GLuint tex, RefCountedGLsync fence);
+               ~OutputChannel();
+               void output_frame(DisplayFrame frame);
                bool get_display_frame(DisplayFrame *frame);
                void set_frame_ready_callback(new_frame_ready_callback_t callback);
-               void set_size(int width, int height);  // Ignored for OUTPUT_LIVE.
 
        private:
                friend class Mixer;
@@ -126,8 +132,6 @@ private:
                bool has_current_frame = false, has_ready_frame = false;  // protected by <frame_mutex>
                new_frame_ready_callback_t new_frame_ready_callback;
                bool has_new_frame_ready_callback = false;
-
-               int width = 1280, height = 720;
        };
        OutputChannel output_channel[NUM_OUTPUTS];